/*
 jGuard is a security framework based on top of jaas (java authentication and authorization security).
 it is written for web applications, to resolve simply, access control problems.
 version $Name$
 http://sourceforge.net/projects/jguard/

 Copyright (C) 2004  Charles GAY

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 jGuard project home page:
 http://sourceforge.net/projects/jguard/

 */
package net.sf.jguard.ext.authentication.loginmodules;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.URL;
import java.security.NoSuchProviderException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;

import net.sf.jguard.ext.SecurityConstants;
import net.sf.jguard.ext.authentication.callbacks.CertificatesCallback;

import org.bouncycastle.ocsp.BasicOCSPResp;
import org.bouncycastle.ocsp.CertificateID;
import org.bouncycastle.ocsp.OCSPException;
import org.bouncycastle.ocsp.OCSPReq;
import org.bouncycastle.ocsp.OCSPReqGenerator;
import org.bouncycastle.ocsp.OCSPResp;
import org.bouncycastle.ocsp.SingleResp;

/**
 *
 * Check the revocation status of a public key certificate using <acronym title="Online Certificate Status Protocol">OCSP</acronym>.
 * this class comes from the EJBCA project. the <a href="http://ejbca.sourceforge.net/">EJBCA</a> project is released under the
 * <a href="http://www.gnu.org/copyleft/lesser.html">LGPL</a> licence. licence, like the jGuard project.
 *
 * @author <a href="mailto:simon.lebettre[at)gmail.com">Simon Lebettre</a>
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 *
 */
public class OCSPLoginModule implements LoginModule {

	private static final String X509 = "X509";
	private static final String CONTENT_TYPE = "Content-Type";
	private static final String APPLICATION_OCSP_REQUEST = "application/ocsp-request";
	private static final String POST = "POST";
	private static final String BC = "BC";
	/** Logger for this class */
	private static final Logger logger = Logger.getLogger(OCSPLoginModule.class.getName());
	private Map sharedState;
	private boolean debug = false;
	private CallbackHandler callbackHandler;
	private Map options;
	private Subject subject;
	private URL ocspServerUrl;
	private X509Certificate issuerCACert;
	private String issuerCACertLocation;
	private X509Certificate OcspSignerCert;
	private String OcspSignerCertLocation;
	private Object certStatus = null;
	private X509Certificate[] certChainToCheck;
	private static boolean SecurityProviderInitialized = false;

	/**
	 * initialize the LoginModule with the required issuer, the OCSP server certificates and its adress.
	 *
	 * @see javax.security.auth.spi.LoginModule#initialize(javax.security.auth.Subject,
	 *      javax.security.auth.callback.CallbackHandler, java.util.Map, java.util.Map)
	 */
	public void initialize(Subject subj, CallbackHandler cbkHandler, Map sState, Map opts) {
		this.subject = subj;
		this.callbackHandler = cbkHandler;
		this.sharedState = sState;
		this.options = opts;

		if (!SecurityProviderInitialized) {
			SecurityProviderInitialized = CRLLoginModule.initSecurityProvider();
		}

		try {
			ocspServerUrl = new URL((String) options.get(SecurityConstants.OCSP_SERVER_URL));
		} catch (MalformedURLException e) {
			logger.severe("ocspServerUrl=" + ocspServerUrl + " is malformed");
		}
		issuerCACertLocation = (String) options.get(SecurityConstants.ISSUER_CA_CERT_LOCATION);
		try {
			issuerCACert = getCertFromFile(issuerCACertLocation);
			OcspSignerCertLocation = (String) options.get(SecurityConstants.OCSP_SIGNER_CERT_LOCATION);
			OcspSignerCert = getCertFromFile(OcspSignerCertLocation);
		} catch (CertificateException e) {
			logger.log(Level.SEVERE, "", e);
		}

		if (!issuerCACert.equals(OcspSignerCert)) {
			throw new UnsupportedOperationException("Having a CA cert different "
					+ "from ocspSigner cert is not currently supported," + " the ocsp response is signed by the CA ");
		}

	}

	/**
	 *
	 * verify either user is registered or not.
	 *
	 * @see javax.security.auth.spi.LoginModule#login()
	 */
	public boolean login() throws LoginException {
		if (callbackHandler == null) {
			throw new LoginException("there is no CallbackHandler to authenticate the user");
		}
		Callback[] callbacks = new Callback[1];
		callbacks[0] = new CertificatesCallback();
		try {
			callbackHandler.handle(callbacks);
		} catch (IOException e1) {
			logger.log(Level.SEVERE, " IOException when we handle callbacks with callback " + callbackHandler.getClass().getName(), e1);
		} catch (UnsupportedCallbackException e1) {
			logger.log(Level.SEVERE, " one callback type is not supported ", e1);
		}
		certChainToCheck = ((CertificatesCallback) callbacks[0]).getCertificates();
		if (certChainToCheck == null) {
			throw new LoginException(" there is no certificates to check with the OCSP protocol ");
		}
		byte[] ocspRequest;
		try {
			OCSPResp response = null;
			try {
				ocspRequest = generateOcspRequest(certChainToCheck);
				byte[] respBytes = getResponseFromHttp(ocspRequest, ocspServerUrl);
				response = new OCSPResp(new ByteArrayInputStream(respBytes));
			} catch (IOException e) {
				logger.log(Level.SEVERE, " IOException when we build the OCSPResponse from HTTP ", e);

			}

			BasicOCSPResp brep = (BasicOCSPResp) response.getResponseObject();
			X509Certificate[] chain = brep.getCerts(OCSPLoginModule.BC);
			boolean verify = brep.verify(chain[0].getPublicKey(), OCSPLoginModule.BC);

			if (!verify) {
				throw new LoginException(" OCSP response is not valid ");
			}

			SingleResp[] singleResps = brep.getResponses();
			for (int i = 0; i < singleResps.length; i++) {
				SingleResp singleResp = singleResps[i];
				certStatus = singleResp.getCertStatus();
				// when the status object is null, the response is good (@see org.bouncycastle.ocsp.SingleResp )

				if (certStatus == null) {
					continue;
				} else {
					throw new LoginException(" status is not null. 'null' is the success result " + certStatus.toString());
				}
			}

		} catch (OCSPException e) {
			throw new LoginException(e.getMessage());
		} catch (NoSuchProviderException e) {
			throw new LoginException(e.getMessage());
		}

		return true;

	}

	/**
	 * ask from the OCSP server if the certificate is valid.
	 *
	 * @param ocspPackage
	 * @param url
	 * @return response from the OCSP server.
	 * @throws IOException
	 * @throws IOException
	 * @throws ProtocolException
	 */
	private byte[] getResponseFromHttp(byte[] ocspPackage, URL url) throws IOException {
		HttpURLConnection con = (HttpURLConnection) url.openConnection();

		con.setDoOutput(true);
		try {
			con.setRequestMethod(OCSPLoginModule.POST);
		} catch (ProtocolException e) {
			throw new IOException(e.getMessage());
		}

		con.setRequestProperty(OCSPLoginModule.CONTENT_TYPE, OCSPLoginModule.APPLICATION_OCSP_REQUEST);
		OutputStream os = con.getOutputStream();
		os.write(ocspPackage);
		os.close();

		ByteArrayOutputStream baos = new ByteArrayOutputStream();

		// This works for small requests, and OCSP requests are small
		InputStream in = con.getInputStream();
		int b = in.read();
		while (b != -1) {
			baos.write(b);
			b = in.read();
		}
		baos.flush();
		in.close();
		byte[] respBytes = baos.toByteArray();
		return respBytes;

	}

	/**
	 * generate the OCSP request.
	 * @param certsToCheck
	 * @return byte[] ocsp request
	 * @throws OCSPException
	 * @throws IOException
	 */
	private byte[] generateOcspRequest(X509Certificate[] certsToCheck) throws OCSPException, IOException {
		OCSPReqGenerator gen = new OCSPReqGenerator();
		for (int i = 0; i < certsToCheck.length; i++) {
			X509Certificate certToCheck = certsToCheck[i];
			gen.addRequest(new CertificateID(CertificateID.HASH_SHA1, issuerCACert, certToCheck.getSerialNumber()));
		}
		OCSPReq req = gen.generate();
		byte[] ocspRequest = req.getEncoded();
		return ocspRequest;
	}

	/**
	 * commit in Subject informations grabeed from X509 certificates.
	 *
	 * @see javax.security.auth.spi.LoginModule#commit()
	 */
	public boolean commit() throws LoginException {
		return CRLLoginModule.certificateCommit(this.subject, this.certChainToCheck);
	}

	/**
	 *
	 * @see javax.security.auth.spi.LoginModule#abort()
	 */
	public boolean abort() throws LoginException {
		return false;
	}

	/**
	 *
	 * @see javax.security.auth.spi.LoginModule#logout()
	 */
	public boolean logout() throws LoginException {
		return false;
	}

	/**
	 * Read a certificate from the specified filepath.
	 *
	 * @param path
	 * @return certificate
	 * @throws CertificateException
	 */
	private static X509Certificate getCertFromFile(String path) throws CertificateException {
		X509Certificate cert = null;

		File certFile = new File(path);
		if (!certFile.canRead()) {
			logger.severe(" File " + certFile.toString() + " is unreadable");
			throw new CertificateException(" File " + certFile.toString() + " is unreadable");

		}
		FileInputStream fis = null;
		try {
			fis = new FileInputStream(path);
		} catch (FileNotFoundException e) {
			logger.log(Level.SEVERE, "we cannot found the certificate file here:" + path, e);
		}
		CertificateFactory cf;
		cf = CertificateFactory.getInstance(OCSPLoginModule.X509);
		cert = (X509Certificate) cf.generateCertificate(fis);

		return cert;
	}

}
