/*
 jGuard is a security framework based on top of jaas (java authentication and authorization security).
 it is written for web applications, to resolve simply, access control problems.
 version $Name$
 http://sourceforge.net/projects/jguard/

 Copyright (C) 2004  Charles GAY

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 jGuard project home page:
 http://sourceforge.net/projects/jguard/

 */
package net.sf.jguard.ext.authentication.manager;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.security.Principal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.security.auth.Subject;

import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.authentication.credentials.JGuardCredential;
import net.sf.jguard.core.principals.RolePrincipal;
import net.sf.jguard.ext.SecurityConstants;
import net.sf.jguard.ext.authentication.AuthenticationException;
import net.sf.jguard.ext.authentication.SubjectTemplate;
import net.sf.jguard.ext.database.ConnectionFactory;
import net.sf.jguard.ext.database.DatabaseUtils;
import net.sf.jguard.ext.principals.PrincipalUtils;
import net.sf.jguard.ext.util.XMLUtils;

/**
 * AuthenticationManager implementation providing a database backend. differences between database "dialects", are handled through
 * specific properties files loaded during the init method.these properties files contains specific SQL queries.
 *
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 * @author <a href="mailto:tandilero@users.sourceforge.net">Maximiliano Batelli</a>
 * @author <a href="mailto:vberetti@users.sourceforge.net">Vincent Beretti</a>
 * @see AuthenticationManager
 */
public class JdbcAuthenticationManager extends AbstractAuthenticationManager implements AuthenticationManager {

	private static final Logger logger = Logger.getLogger(JdbcAuthenticationManager.class.getName());
	private Properties properties = null;
	private String dbPropertiesLocation = null;
	private ConnectionFactory connectionFactory = null;

	public JdbcAuthenticationManager() {
		super();
	}

	protected void persistPrincipal(Principal principal) throws AuthenticationException {
		Connection connection = connectionFactory.getConnection();
		try {
			PreparedStatement pst = connection.prepareStatement(properties.getProperty("ADD_PRINCIPAL"));
			pst.setString(1, principal.getName());
			if (principal instanceof RolePrincipal) {
				pst.setString(2, ((RolePrincipal) principal).getApplicationName());
			} else {
				pst.setString(2, applicationName);
			}
			pst.setString(3, principal.getClass().getName());
			pst.executeUpdate();
			pst.close();

			localPrincipals.put(principal.getName(), principal);
			localPrincipalsSet.add(principal);

		} catch (SQLException e) {
			logger.log(Level.SEVERE, " persistPrincipal(Principal) ", e);
			throw new AuthenticationException(e);
		} finally {
			try {
				connection.close();
			} catch (SQLException e) {
				logger.log(Level.SEVERE, " persistPrincipal(Principal) ", e);
				throw new AuthenticationException(e);
			}
		}

	}

	public void init(Map map) {

		dbPropertiesLocation = (String) map.get(SecurityConstants.AUTHENTICATION_DATABASE_FILE_LOCATION);
		dbPropertiesLocation = XMLUtils.resolveLocation(dbPropertiesLocation);
		properties = new Properties();
		try {
			properties.load(new FileInputStream(new File(new URI(dbPropertiesLocation))));
		} catch (FileNotFoundException e) {
			logger.severe(" authorization database properties file is not found at this location " + dbPropertiesLocation);
		} catch (IOException e) {
			logger.severe(" authorization database properties file is not accesible this location " + dbPropertiesLocation + "\n "
					+ e.getMessage());
		} catch (URISyntaxException e) {
			logger.log(Level.SEVERE, " uri of the authorization database properties file hasn't got a valid synthax ", e);
		}

		logger.finest("JdbcAuthenticationManager connectionFactory init map =" + map);
		logger.finest("JdbcAuthenticationManager connectionFactory properties=" + properties);
		// initialize database connection factory
		connectionFactory = new ConnectionFactory(map);
		super.applicationName = (String) map.get(CoreConstants.APPLICATION_NAME);
		createRequiredDatabaseEntities(properties, this.connectionFactory);

		boolean empty = isEmpty();
		if (empty) {
			insertRequiredData();
		}

		// we don't grab all the users like in XMLAuthenticationManager,
		// because XML is only used for small applications
		// it will be not suited to load all the users in memory

		try {
			initPrincipals();
		} catch (SQLException e) {
			logger.log(Level.SEVERE, " principals cannot be initialized ", e);
		}

		try {
			subjectTemplate = getSubjectTemplate(DEFAULT);
		} catch (AuthenticationException e) {
			logger.log(Level.SEVERE, " default subjectTemplate cannot be initialized ", e);
		}

	}

	private static void createRequiredDatabaseEntities(Properties properties, ConnectionFactory connectionFactory) {
		List tablesNames = new ArrayList();
		tablesNames.add("JG_PRINCIPAL");
		tablesNames.add("JG_USER");
		tablesNames.add("JG_CREDENTIAL");
		tablesNames.add("JG_USER_PRINCIPAL");

		tablesNames.add("JG_ST_CREDENTIAL");
		tablesNames.add("JG_ST_PRINCIPAL");
		tablesNames.add("JG_ST_USER");

		List foreignkeysNames = new ArrayList();
		foreignkeysNames.add("FK_USER_USER_PRINCIPAL");
		foreignkeysNames.add("FK_USER_PRINCIPAL");
		foreignkeysNames.add("FK_CREDENTIAL_USER_ID");

		foreignkeysNames.add("FK_ST_USER_PRINCIPAL");
		foreignkeysNames.add("FK_ST_USER_CREDENTIAL");

		List sequencesNames = new ArrayList();
		sequencesNames.add("JG_PRINCIPAL_SEQ");
		sequencesNames.add("JG_USER_SEQ");
		sequencesNames.add("JG_CREDENTIAL_SEQ");

		sequencesNames.add("JG_ST_CREDENTIAL_SEQ");
		sequencesNames.add("JG_ST_PRINCIPAL_SEQ");
		sequencesNames.add("JG_ST_USER_SEQ");
		// initialise database
		DatabaseUtils
				.createRequiredDatabaseEntities(properties, connectionFactory, sequencesNames, tablesNames, foreignkeysNames);
	}

	/**
	 * initialize Principals declared in the application, and principals owned by the default
	 * @throws SQLException
	 * @see SubjectTemplate
	 */
	private void initPrincipals() throws SQLException {
		Connection connection = connectionFactory.getConnection();
		PreparedStatement pst = connection.prepareStatement(properties.getProperty("LOCAL_PRINCIPALS"));
		pst.setString(1, this.applicationName);
		ResultSet rs = pst.executeQuery();
		while (rs.next()) {
			String name = rs.getString(2);
			String applicationName = rs.getString(3);
			String className = rs.getString(4);
			Principal principal = PrincipalUtils.getPrincipal(className, name);
			if (principal instanceof RolePrincipal) {
				((RolePrincipal) principal).setApplicationName(applicationName);
			}
			localPrincipalsSet.add(principal);
			localPrincipals.put(principal.getName(), principal);
		}

		rs.close();
		pst.close();
	}

	private void insertRequiredData() {
		Map options = new HashMap();
		options.put(CoreConstants.APPLICATION_NAME, applicationName);
		String XmlFileLocation = dbPropertiesLocation.substring(0, dbPropertiesLocation.lastIndexOf('/'))
				+ "/jGuardUsersPrincipals.xml";
		options.put(SecurityConstants.AUTHENTICATION_XML_FILE_LOCATION, XmlFileLocation);
		AuthenticationManager authentManager = new XmlAuthenticationManager();
		authentManager.init(options);
		importAuthenticationManager(authentManager);

	}

	public boolean isEmpty() {
		List selectQueries = new ArrayList();
		selectQueries.add("PRINCIPALS");
		selectQueries.add("CREDENTIALS");
		selectQueries.add("CREDENTIALS");
		selectQueries.add("USERS");

		return DatabaseUtils.isEmpty(this.properties, connectionFactory, selectQueries);

	}

	protected void updateUserImpl(JGuardCredential cred, Subject user) throws AuthenticationException {

		Connection connection = connectionFactory.getConnection();
		try {
			long subjectId = getSubjectId(cred, user);

			// get user from DB
			Subject oldUser = getUser(subjectId);

			connection.setAutoCommit(false);

			updatePrivateCredentials(connection, subjectId, user, oldUser);
			updatePublicCredentials(connection, subjectId, user, oldUser);
			updatePrincipals(connection, subjectId, user, oldUser);

			connection.commit();
			connection.setAutoCommit(true);
		} catch (SQLException e) {
			logger.log(Level.SEVERE, " user can not be updated ", e);
			if (connection != null)
				try {
					connection.rollback();
				} catch (SQLException excep) {
					throw new AuthenticationException(e);
				}
			throw new AuthenticationException(e);
		} catch (SecurityException e) {
			logger.log(Level.FINEST, " user can not be updated: you don't have the permission to update private credentials", e);
			if (connection != null)
				try {
					connection.rollback();
				} catch (SQLException excep) {
					throw new AuthenticationException(excep);
				}
			throw new AuthenticationException(e);
		} finally {
			try {
				connection.close();
			} catch (SQLException e) {
				logger.log(Level.SEVERE, " user can not be updated ", e);
				throw new AuthenticationException(e);
			}
		}

	}

	//TODO think better how to deal with definition in RolePrincipal, because if we only update the definition attr
	// the RolePrincipal is not updated with this method
	// by now, we are deleting and creating again all user principals
	/*
	private void updatePrincipals(Connection connection, long subjectId, Subject user, Subject oldUser) throws SQLException {
		Set deletedPrincipals = new HashSet(oldUser.getPrincipals());
		deletedPrincipals.removeAll(user.getPrincipals());
		unlinkUserFromPrincipals(connection, subjectId, deletedPrincipals);

		Set newPrincipals = new HashSet(user.getPrincipals());
		newPrincipals.removeAll(oldUser.getPrincipals());
		linkUserToPrincipals(connection, subjectId, newPrincipals);
	}
	*/

	private void updatePrincipals(Connection connection, long subjectId, Subject user, Subject oldUser) throws SQLException {
		unlinkUserFromPrincipals(connection, subjectId, oldUser.getPrincipals());
		linkUserToPrincipals(connection, subjectId, user.getPrincipals());
	}
	
	private long getPrincipalId(Principal principal) throws SQLException {

		Connection connection = connectionFactory.getConnection();
		PreparedStatement pst = connection.prepareStatement(properties.getProperty("GET_PRINCIPAL_ID"));
		long principalId = -1;
		pst.setString(1, principal.getName());
		if (principal instanceof RolePrincipal) {
			pst.setString(2, ((RolePrincipal) principal).getApplicationName());
		} else {
			pst.setString(2, applicationName);
		}
		ResultSet rs = pst.executeQuery();
		rs.next();
		principalId = rs.getLong(1);
		rs.close();
		pst.close();
		connection.close();

		return principalId;
	}

	private void unlinkUserFromPrincipals(Connection connection, long subjectId, Set principals) throws SQLException {

		PreparedStatement pst = connection.prepareStatement(properties.getProperty("UNLINK_USER_PRINCIPAL"));
		Iterator it = principals.iterator();
		while (it.hasNext()) {
			Principal principal = (Principal) it.next();
			long principalId = getPrincipalId(principal);
			pst.setLong(1, subjectId);
			pst.setLong(2, principalId);
			pst.executeUpdate();
		}
		pst.close();
	}

	private void linkUserToPrincipals(Connection connection, long subjectId, Set principals) throws SQLException {
		PreparedStatement pst = connection.prepareStatement(properties.getProperty("LINK_USER_PRINCIPAL"));
		Iterator it = principals.iterator();
		while (it.hasNext()) {
			Principal principal = (Principal) it.next();
			long principalId = getPrincipalId(principal);
			pst.setLong(1, subjectId);
			pst.setLong(2, principalId);
			if(principal instanceof RolePrincipal) {
				pst.setString(3, ((RolePrincipal)principal).getDefinition());
				pst.setBoolean(4, ((RolePrincipal)principal).isActive());
			}
			pst.executeUpdate();
		}
		pst.close();
	}

	private void updatePrivateCredentials(Connection connection, long subjectId, Subject user, Subject oldUser)
			throws SQLException {

		// delete old private credentials
		Set deletedPrivateCredentials = new HashSet(oldUser.getPrivateCredentials(JGuardCredential.class));
		deletedPrivateCredentials.removeAll(user.getPrivateCredentials(JGuardCredential.class));
		Iterator itDeletedPrivateCredentials = deletedPrivateCredentials.iterator();
		while (itDeletedPrivateCredentials.hasNext()) {
			deleteCredential(connection, (JGuardCredential) itDeletedPrivateCredentials.next(), subjectId);
		}

		// add new private credentials
		Set newPrivateCredentials = new HashSet(user.getPrivateCredentials(JGuardCredential.class));
		newPrivateCredentials.removeAll(oldUser.getPrivateCredentials(JGuardCredential.class));
		Iterator itNewPrivateCredentials = newPrivateCredentials.iterator();
		while (itNewPrivateCredentials.hasNext()) {
			persistCredential(connection, (JGuardCredential) itNewPrivateCredentials.next(), subjectId, false);
		}
	}

	private void updatePublicCredentials(Connection connection, long subjectId, Subject user, Subject oldUser)
			throws SQLException {

		// delete old public credentials
		Set deletedPublicCredentials = new HashSet(oldUser.getPublicCredentials(JGuardCredential.class));
		deletedPublicCredentials.removeAll(user.getPublicCredentials(JGuardCredential.class));
		Iterator itDeletedPublicCredentials = deletedPublicCredentials.iterator();
		while (itDeletedPublicCredentials.hasNext()) {
			deleteCredential(connection, (JGuardCredential) itDeletedPublicCredentials.next(), subjectId);
		}

		// add new public credentials
		Set newPublicCredentials = new HashSet(user.getPublicCredentials(JGuardCredential.class));
		newPublicCredentials.removeAll(oldUser.getPublicCredentials(JGuardCredential.class));
		Iterator itNewPublicCredentials = newPublicCredentials.iterator();
		while (itNewPublicCredentials.hasNext()) {
			persistCredential(connection, (JGuardCredential) itNewPublicCredentials.next(), subjectId, true);
		}
	}

	public void deleteUser(Subject user) throws AuthenticationException {
		Connection connection = connectionFactory.getConnection();
		PreparedStatement pst = null;

		try {
			connection.setAutoCommit(false);
			long subjectId = getSubjectId(user);

			deleteCredentials(connection, subjectId);
			deletePrincipals(connection, subjectId);

			pst = connection.prepareStatement(properties.getProperty("DELETE_SUBJECT"));
			pst.setLong(1, subjectId);
			pst.executeUpdate();
			pst.close();

			connection.commit();
			connection.setAutoCommit(true);

		} catch (SQLException e) {
			logger.log(Level.SEVERE, "deleteUser(Subject user)", e);
			if (connection != null)
				try {
					connection.rollback();
				} catch (SQLException excep) {
					throw new AuthenticationException(excep);
				}
			throw new AuthenticationException(e);
		} finally {
			try {
				connection.close();
			} catch (SQLException e) {
				logger.log(Level.SEVERE, "deleteUser(Subject user)", e);
				throw new AuthenticationException(e);
			}
		}

	}

	private long getSubjectId(Subject user) throws SQLException, AuthenticationException {
		JGuardCredential identityCred = extractIdentityCredentialFromUser(user);
		return getSubjectId(identityCred, user);
	}

	private long getSubjectId(JGuardCredential identityCred, Subject user) throws SQLException, AuthenticationException {
		long userId = -1;
		Connection connection = connectionFactory.getConnection();
		PreparedStatement pst = connection.prepareStatement(properties.getProperty("GET_USER_ID"));
		pst.setString(1, identityCred.getId());
		pst.setString(2, (String) identityCred.getValue());

		ResultSet rs = pst.executeQuery();
		while (rs.next()) {
			if (userId != -1) {
				logger.severe("it exists more than one user with this identity credential ");
				throw new AuthenticationException("it exists more than one user with this identity credential ");
			}
			userId = rs.getLong(1);
		}
		if (userId == -1) {
			logger.severe("no user exists with this identity credential ");
			throw new AuthenticationException("no user exists with this identity credential ");
		}

		rs.close();
		pst.close();
		connection.close();

		return userId;
	}

	private void deletePrincipals(Connection connection, long subjectId) throws SQLException {
		PreparedStatement pst = null;
		pst = connection.prepareStatement(properties.getProperty("DELETE_USER_PRINCIPALS"));
		pst.setLong(1, subjectId);
		pst.executeUpdate();
		pst.close();
	}

	private void deleteCredentials(Connection connection, long subjectId) throws SQLException {
		PreparedStatement pst = null;
		pst = connection.prepareStatement(properties.getProperty("DELETE_USER_CREDENTIALS"));
		pst.setLong(1, subjectId);
		pst.executeUpdate();
		pst.close();
	}

	private void persistCredentials(Connection connection, Subject subject, long subjectId) throws SQLException {
		Set publicCredentials = subject.getPublicCredentials();
		Iterator itPublic = publicCredentials.iterator();
		while (itPublic.hasNext()) {
			JGuardCredential jgCred = (JGuardCredential) itPublic.next();
			persistCredential(connection, jgCred, subjectId, true);
		}

		Set privateCredentials = subject.getPrivateCredentials();
		Iterator itPrivate = privateCredentials.iterator();
		while (itPrivate.hasNext()) {
			JGuardCredential jgCred = (JGuardCredential) itPrivate.next();
			persistCredential(connection, jgCred, subjectId, false);
		}

	}

	private void persistCredential(Connection connection, JGuardCredential jgCred, long userId, boolean publicVisibility)
			throws SQLException {

		PreparedStatement pst = connection.prepareStatement(properties.getProperty("ADD_CREDENTIAL"));
		pst.setLong(1, userId);
		pst.setBoolean(2, publicVisibility);
		pst.setString(3, jgCred.getId());
		pst.setString(4, (String) jgCred.getValue());
		pst.executeUpdate();
		pst.close();
	}

	// TODO VBE : remove this method. Credentials are not updated anymore, they are removed and added.
	// This is done because this method is not handling multivalued credentials
	private void updateCredential(Connection connection, JGuardCredential jgCred, long userId) throws SQLException {

		PreparedStatement pst = connection.prepareStatement(properties.getProperty("UPDATE_CREDENTIAL"));
		pst.setString(1, (String) jgCred.getValue());
		pst.setLong(2, userId);
		pst.setString(3, jgCred.getId());
		pst.executeUpdate();
		pst.close();
	}

	private void deleteCredential(Connection connection, JGuardCredential jgCred, long userId) throws SQLException {

		PreparedStatement pst = connection.prepareStatement(properties.getProperty("DELETE_CREDENTIAL"));
		pst.setLong(1, userId);
		pst.setString(2, jgCred.getId());
		pst.setString(3, (jgCred.getValue() != null ? jgCred.getValue().toString() : ""));
		pst.executeUpdate();
		pst.close();
	}

	private void persistPrincipals(Connection connection, Subject subject, long subjectId) throws SQLException,
			AuthenticationException {

		PreparedStatement pst = connection.prepareStatement(properties.getProperty("LINK_USER_PRINCIPAL"));
		Set principals = subject.getPrincipals();
		Iterator it = principals.iterator();
		while (it.hasNext()) {
			Principal ppal = (Principal) it.next();
			createPrincipal(ppal);

			long principalId = getPrincipalId(ppal);

			pst.setLong(1, subjectId);
			pst.setLong(2, principalId);
			if(ppal instanceof RolePrincipal) {
				pst.setString(3, ((RolePrincipal)ppal).getDefinition());
				pst.setBoolean(4, ((RolePrincipal)ppal).isActive());
			}
			
			pst.executeUpdate();
		}
		pst.close();
	}

	protected void persistUser(Subject user) throws AuthenticationException {
		Connection connection = connectionFactory.getConnection();
		PreparedStatement pst = null;
		PreparedStatement pst2 = null;
		ResultSet rs2 = null;

		try {
			connection.setAutoCommit(false);

			pst = connection.prepareStatement(properties.getProperty("ADD_SUBJECT"));
			pst.executeUpdate();
			pst.close();

			pst2 = connection.prepareStatement(properties.getProperty("GET_SUBJECT_ID"));
			rs2 = pst2.executeQuery();
			rs2.next();
			long subjectId = rs2.getLong(1);
			rs2.close();
			pst2.close();

			persistCredentials(connection, user, subjectId);
			persistPrincipals(connection, user, subjectId);

			connection.commit();
			connection.setAutoCommit(true);

		} catch (SQLException e) {
			logger.log(Level.SEVERE, "persistUser(Subject user)", e);
			if (connection != null)
				try {
					connection.rollback();
				} catch (SQLException excep) {
					throw new AuthenticationException(excep);
				}
			throw new AuthenticationException(e);
		} finally {
			try {
				connection.close();
			} catch (SQLException e) {
				logger.log(Level.SEVERE, "persistUser(Subject user)", e);
				throw new AuthenticationException(e);
			}
		}

	}

	public void updateDefaultSubjectTemplate(SubjectTemplate template) {
		// TODO Auto-generated method stub

	}

	public void persistSubjectTemplate(SubjectTemplate template) throws AuthenticationException {
		Connection connection = connectionFactory.getConnection();
		long stId = -1;
		try {
			connection.setAutoCommit(false);

			PreparedStatement pst = connection.prepareStatement(properties.getProperty("ADD_SUBJECT_TEMPLATE"));
			pst.setString(1, template.getName());
			pst.executeUpdate();
			pst.close();

			PreparedStatement pst2 = connection.prepareStatement(properties.getProperty("GET_SUBJECT_TEMPLATE_ID"));
			ResultSet rs2 = pst2.executeQuery();
			rs2.next();
			stId = rs2.getLong(1);
			rs2.close();
			pst2.close();

			persistSubjectTemplateCredentials(connection, template, stId);
			persistSubjectTemplatePrincipals(connection, template, stId);

			connection.commit();
			connection.setAutoCommit(true);
		} catch (SQLException e) {
			logger.log(Level.SEVERE, " subject_template cannot be created ", e);
			if (connection != null)
				try {
					connection.rollback();
				} catch (SQLException excep) {
					throw new AuthenticationException(excep);
				}
			throw new AuthenticationException(e);
		} finally {
			try {
				connection.close();
			} catch (SQLException e) {
				logger.log(Level.SEVERE, "subject_template cannot be created", e);
				throw new AuthenticationException(e);
			}
		}
	}

	private void persistSubjectTemplatePrincipals(Connection connection, SubjectTemplate template, long stId) throws SQLException {

		Set genericPrincipals = template.getPrincipals();
		Iterator itGenPrincipals = genericPrincipals.iterator();
		while (itGenPrincipals.hasNext()) {
			Principal ppal = (Principal) itGenPrincipals.next();
			persistSubjectTemplatePrincipal(connection, ppal, stId);
		}
	}

	private void persistSubjectTemplatePrincipal(Connection connection, Principal principal, long stId) throws SQLException {

		PreparedStatement pst = connection.prepareStatement(properties.getProperty("ADD_ST_PRINCIPAL"));
		pst.setString(1, principal.getName());
		if (principal instanceof RolePrincipal) {
			pst.setString(2, ((RolePrincipal) principal).getApplicationName());
		} else {
			pst.setString(2, applicationName);
		}
		pst.setString(3, principal.getClass().getName());

		pst.setLong(4, stId);
		pst.executeUpdate();
		pst.close();

		localPrincipals.put(applicationName + "#" + principal.getName(), principal);
		localPrincipalsSet.add(principal);
	}

	private void persistSubjectTemplateCredentials(Connection connection, SubjectTemplate st, long subjectTemplateId)
			throws SQLException {

		Set privateRequiredCred = st.getPrivateRequiredCredentials();
		Iterator itPrivateRequiredCred = privateRequiredCred.iterator();
		while (itPrivateRequiredCred.hasNext()) {
			JGuardCredential jgCred = (JGuardCredential) itPrivateRequiredCred.next();
			// required and public: 'true' and 'false'
			persistStCredential(connection, jgCred, subjectTemplateId, true, false);
		}

		Set publicRequiredCreds = st.getPublicRequiredCredentials();
		Iterator itPublicRequiredCred = publicRequiredCreds.iterator();
		while (itPublicRequiredCred.hasNext()) {
			JGuardCredential jgCred = (JGuardCredential) itPublicRequiredCred.next();
			// required and public: both 'true'
			persistStCredential(connection, jgCred, subjectTemplateId, true, true);
		}

		Set privateOptionalCred = st.getPrivateOptionalCredentials();
		Iterator itPrivateOptionalCred = privateOptionalCred.iterator();
		while (itPrivateOptionalCred.hasNext()) {
			JGuardCredential jgCred = (JGuardCredential) itPrivateOptionalCred.next();
			// required and public: both 'false'
			persistStCredential(connection, jgCred, subjectTemplateId, false, false);
		}

		Set publicOptionalCreds = st.getPublicOptionalCredentials();
		Iterator itPublicOptionalCred = publicOptionalCreds.iterator();
		while (itPublicOptionalCred.hasNext()) {
			JGuardCredential jgCred = (JGuardCredential) itPublicOptionalCred.next();
			// required and public: 'false' and 'true'
			persistStCredential(connection, jgCred, subjectTemplateId, false, true);
		}
	}

	private void persistStCredential(Connection connection, JGuardCredential jgCred, long subjectTemplateId, boolean required,
			boolean public_visibility) throws SQLException {

		PreparedStatement pst = connection.prepareStatement(properties.getProperty("ADD_ST_CREDENTIAL"));
		pst.setString(1, jgCred.getId());
		pst.setBoolean(2, public_visibility);
		pst.setBoolean(3, required);
		pst.setBoolean(4, jgCred.isIdentity());
		pst.setLong(5, subjectTemplateId);
		pst.executeUpdate();
		pst.close();
	}

	public final ConnectionFactory getConnectionFactory() {
		return connectionFactory;
	}

	public final Properties getProperties() {
		return properties;
	}

	public final void setProperties(Properties properties) {
		this.properties = properties;
	}

	public SubjectTemplate getSubjectTemplate(String name) throws AuthenticationException {
		Connection connection = connectionFactory.getConnection();
		PreparedStatement pst = null;
		PreparedStatement pst2 = null;
		PreparedStatement pst3 = null;
		SubjectTemplate st = new SubjectTemplate();
		ResultSet rs = null;
		ResultSet rs2 = null;
		ResultSet rs3 = null;

		try {
			pst = connection.prepareStatement(properties.getProperty("GET_SUBJECT_TEMPLATE"));
			pst.setString(1, name);
			rs = pst.executeQuery();
			long subjectTemplateId = -1;
			while (rs.next()) {
				subjectTemplateId = rs.getLong(1);
			}
			rs.close();
			pst.close();

			pst2 = connection.prepareStatement(properties.getProperty("GET_ST_PRINCIPALS"));
			pst2.setLong(1, subjectTemplateId);
			rs2 = pst2.executeQuery();
			Set genericPrincipals = new HashSet();
			while (rs2.next()) {
				String className = rs2.getString(4);
				String principalName = (String) rs2.getString(2);
				Principal ppal = PrincipalUtils.getPrincipal(className, principalName);
				if (className.equals(RolePrincipal.class.getName())) {
					((RolePrincipal) ppal).setApplicationName(rs2.getString(3));
				}
				genericPrincipals.add(ppal);
			}

			rs2.close();
			pst2.close();
			st.setPrincipals(genericPrincipals);

			pst3 = connection.prepareStatement(properties.getProperty("GET_ST_CREDENTIALS"));
			pst3.setLong(1, subjectTemplateId);
			rs3 = pst3.executeQuery();
			Set privateOptionalCredentials = new HashSet();
			Set privateRequiredCredentials = new HashSet();
			Set publicOptionalCredentials = new HashSet();
			Set publicRequiredCredentials = new HashSet();

			while (rs3.next()) {
				JGuardCredential cred = new JGuardCredential();
				cred.setId(rs3.getString(2));
				boolean public_visibility = rs3.getBoolean(3);
				boolean required = rs3.getBoolean(4);
				cred.setIdentity(rs3.getBoolean(5));
				if (public_visibility && required) {
					publicRequiredCredentials.add(cred);
				} else if (!public_visibility && required) {
					privateRequiredCredentials.add(cred);
				} else if (public_visibility && !required) {
					publicOptionalCredentials.add(cred);
				} else if (!public_visibility && !required) {
					privateOptionalCredentials.add(cred);
				}

			}
			rs3.close();
			pst3.close();
			st.setPrivateOptionalCredentials(privateOptionalCredentials);
			st.setPrivateRequiredCredentials(privateRequiredCredentials);
			st.setPublicOptionalCredentials(publicOptionalCredentials);
			st.setPublicRequiredCredentials(publicRequiredCredentials);

		} catch (SQLException e) {
			logger.log(Level.SEVERE, " subject_template cannot be grabbed ", e);
			throw new AuthenticationException(e);
		} finally {
			try {
				connection.close();
			} catch (SQLException e) {
				logger.log(Level.SEVERE, "subject_template cannot be grabbed ", e);
				throw new AuthenticationException(e);
			}
		}

		return st;

	}

	public Set getAllPrincipalsSet() throws AuthenticationException {
		Set allPrincipals = new HashSet();
		Connection connection = connectionFactory.getConnection();
		PreparedStatement pst = null;
		ResultSet rs = null;
		try {
			pst = connection.prepareStatement(properties.getProperty("PRINCIPALS"));
			rs = pst.executeQuery();
			while (rs.next()) {
				String name = rs.getString(2);
				String className = rs.getString(4);
				String applicationName = rs.getString(3);
				Principal principal = PrincipalUtils.getPrincipal(className, name);
				if (principal instanceof RolePrincipal) {
					((RolePrincipal) principal).setApplicationName(applicationName);
				}
				allPrincipals.add(principal);
			}

			rs.close();
			pst.close();
		} catch (SQLException e) {
			logger.log(Level.SEVERE, " principals cannot be grabbed ", e);
			throw new AuthenticationException(e);
		} finally {
			try {
				connection.close();
			} catch (SQLException e) {
				logger.log(Level.SEVERE, " principals cannot be grabbed ", e);
				throw new AuthenticationException(e);
			}
		}
		return allPrincipals;
	}

	/**
	 * search the users which matches ALL of the credentials criterions.
	 *
	 * @param credentials
	 *            crierions used to grab the users
	 * @return users found
	 */
	public Set findUsers(Collection credentials) throws AuthenticationException {
		Set users = new HashSet();

		if (credentials.isEmpty())
			return users;

		Connection connection = connectionFactory.getConnection();
		PreparedStatement pst = null;
		ResultSet rs = null;
		String findUsers = properties.getProperty("FIND_USERS");
		String[] tokens = findUsers.split(" ");
		String user_id = tokens[2];
		String jg_credential = tokens[4];
		String cred_name = tokens[6];
		String cred_value = tokens[9];
		try {
			// we build the query
			StringBuffer dynamicQuery = new StringBuffer();
			StringBuffer tables = new StringBuffer();
			StringBuffer conditions = new StringBuffer();

			for (int count = 0; count < credentials.size(); count++) {
				if (count > 0) {
					tables.append(',');
					conditions.append(" and ");
				} else {
					conditions.append(" where ");
				}
				StringBuffer alias = new StringBuffer("cred").append(count);
				tables.append(' ').append(jg_credential).append(' ').append(alias);
				conditions.append(alias).append('.').append(cred_name).append("= ? and ");
				conditions.append(alias).append('.').append(cred_value).append("= ? ");
				if (count > 0) {
					conditions.append(" and cred0.").append(user_id).append('=');
					conditions.append(alias).append('.').append(user_id).append(' ');
				}
			}

			dynamicQuery.append(" select distinct cred0.").append(user_id).append(" from ");
			dynamicQuery.append(tables);
			dynamicQuery.append(conditions);
			logger.finest(dynamicQuery.toString());
			pst = connection.prepareStatement(dynamicQuery.toString());

			Iterator itCredentials = credentials.iterator();
			int count2 = 1;
			while (itCredentials.hasNext()) {
				JGuardCredential jcred = (JGuardCredential) itCredentials.next();
				pst.setString(count2, jcred.getId());
				count2++;
				pst.setString(count2, jcred.getValue().toString());
				count2++;
			}
			rs = pst.executeQuery();
			while (rs.next()) {
				Subject user = null;
				int userId = rs.getInt(1);
				user = getUser(userId);
				users.add(user);
			}

			rs.close();
			pst.close();
		} catch (SQLException e) {
			logger.log(Level.SEVERE, " error when we try to find users which maps to credentials ", e);
			throw new AuthenticationException(e);
		} finally {
			try {
				connection.close();
			} catch (SQLException e) {
				logger.log(Level.SEVERE, "users cannot be found ", e);
				throw new AuthenticationException(e);
			}
		}
		return users;
	}

	/**
	 * search the user's principals .
	 *
	 * @param usrId
	 * @return user principals
	 * @throws SQLException
	 */
	private static Set getUserPrincipals(long usrId) throws SQLException {
		JdbcAuthenticationManager authManager = (JdbcAuthenticationManager) AuthenticationManagerFactory
				.getAuthenticationManager();
		ConnectionFactory connectionFactory = authManager.getConnectionFactory();
		Connection connection = connectionFactory.getConnection();

		PreparedStatement pst = null;
		ResultSet rs = null;
		Set userPrincipals = new HashSet();
		pst = connection.prepareStatement(authManager.getProperties().getProperty("GET_USER_PRINCIPALS"));
		pst.setLong(1, usrId);
		rs = pst.executeQuery();
		while (rs.next()) {
			RolePrincipal principal = new RolePrincipal();
			principal.setName(rs.getString(1));
			principal.setApplicationName(rs.getString(2));
			principal.setDefinition(rs.getString(4));
			principal.setActive(rs.getBoolean(5));
			logger.finest("principal added" + principal + " to user " + usrId);
			userPrincipals.add(principal);
			
		}

		rs.close();
		pst.close();

		return userPrincipals;
	}

	/**
	 * search the user's credentials .
	 *
	 * @param usrId
	 * @param publicVisibility - indicate if the credentials to return are public or private
	 * @return user credentials
	 * @throws SQLException
	 */
	private static Set getUserCredentials(long usrId, boolean publicVisibility) throws SQLException {
		JdbcAuthenticationManager authManager = (JdbcAuthenticationManager) AuthenticationManagerFactory
				.getAuthenticationManager();
		ConnectionFactory connectionFactory = authManager.getConnectionFactory();
		Connection connection = connectionFactory.getConnection();
		PreparedStatement pst = null;
		ResultSet rs = null;
		Set userCredentials = new HashSet();
		pst = connection.prepareStatement(authManager.getProperties().getProperty("GET_USER_CREDENTIALS"));
		pst.setLong(1, usrId);
		pst.setBoolean(2, publicVisibility);
		rs = pst.executeQuery();
		while (rs.next()) {
			JGuardCredential cred = new JGuardCredential();
			cred.setId(rs.getString(1));
			cred.setValue(rs.getString(2));
			userCredentials.add(cred);
			logger.finest(" credential added " + cred + " to user " + usrId);
		}

		rs.close();
		pst.close();

		return userCredentials;
	}

	/**
	 * return all users of jGuard.
	 */
	public Set getUsers() throws AuthenticationException {
		Set users = new HashSet();
		Connection connection = connectionFactory.getConnection();
		PreparedStatement pst = null;
		ResultSet rs = null;
		try {
			pst = connection.prepareStatement(properties.getProperty("GET_USERS"));
			rs = pst.executeQuery();
			while (rs.next()) {
				long userId = rs.getLong(1);
				Subject user = getUser(userId);
				users.add(user);
			}

			rs.close();
			pst.close();
		} catch (SQLException e) {
			logger.log(Level.SEVERE, " users cannot be grabbed ", e);
			throw new AuthenticationException(e);
		} finally {
			try {
				connection.close();
			} catch (SQLException e) {
				logger.log(Level.SEVERE, " users cannot be grabbed ", e);
				throw new AuthenticationException(e);
			}
		}
		return users;
	}

	/**
	 *
	 * @param userId
	 * @return 
	 * @throws SQLException
	 */
	public static Subject getUser(long userId) throws SQLException {
		// SQL exception is not catched because JdbcLoginModule need to have the exception to rollback
		Set userPrincipals = getUserPrincipals(userId);
		Set userPubCred = getUserCredentials(userId, true);
		Set userPrivCred = getUserCredentials(userId, false);
		// subject is not in read-only mode
		Subject user = new Subject(false, userPrincipals, userPubCred, userPrivCred);
		return user;
	}

	public void updatePrincipal(String oldName, Principal principal) throws AuthenticationException {
		if (oldName.equals(principal.getName())) {
			return;
		}
		Connection connection = connectionFactory.getConnection();
		PreparedStatement pst = null;
		try {
			pst = connection.prepareStatement(properties.getProperty("UPDATE_PRINCIPAL"));
			pst.setString(1, principal.getName());
			pst.setString(2, oldName);
			pst.setString(3, applicationName);
			pst.executeUpdate();
			pst.close();

		} catch (SQLException e) {
			logger.log(Level.SEVERE, "updatePrincipal(String oldName, Principal principal)", e);
			throw new AuthenticationException(e);
		} finally {
			try {
				connection.close();
			} catch (SQLException e) {
				logger.log(Level.SEVERE, "updatePrincipal(String oldName, Principal principal)", e);
				throw new AuthenticationException(e);
			}
		}

		// if the database update is right, we can update the in memory reference
		Principal oldPal = (Principal)localPrincipals.remove(oldName);
		localPrincipalsSet.remove(oldPal);
		localPrincipals.put(principal.getName(), principal);
		localPrincipalsSet.add(principal);
		

	}

	public void deletePrincipal(Principal principal) throws AuthenticationException {
		Connection connection = connectionFactory.getConnection();
		PreparedStatement pst = null;
		PreparedStatement pst2 = null;
		PreparedStatement pst3 = null;
		try {
			connection.setAutoCommit(false);

			// we are looking for the principalId
			pst = connection.prepareStatement(properties.getProperty("GET_PRINCIPAL_ID"));
			pst.setString(1, principal.getName());
			pst.setString(2, applicationName);
			ResultSet rs = pst.executeQuery();
			long principalId = -1;
			if (rs.next())
				principalId = rs.getLong(1);
			else
				logger.finest(" there is no principal to delete: " + principal.getName());
			rs.close();
			pst.close();

			// we use principalId to delete links between users and this principal
			pst2 = connection.prepareStatement(properties.getProperty("DELETE_PRINCIPAL_USERS"));
			pst2.setLong(1, principalId);
			pst2.executeUpdate();
			pst2.close();

			// we delete the principal
			pst3 = connection.prepareStatement(properties.getProperty("DELETE_PRINCIPAL"));
			pst3.setString(1, principal.getName());
			pst3.executeUpdate();
			pst3.close();

			connection.commit();
			connection.setAutoCommit(true);
		} catch (SQLException e) {
			logger.log(Level.SEVERE, "deletePrincipal(Principal principal)", e);
			if (connection != null)
				try {
					connection.rollback();
				} catch (SQLException excep) {
					throw new AuthenticationException(e);
				}
			throw new AuthenticationException(e);
		} finally {
			try {
				connection.close();
			} catch (SQLException e) {
				logger.log(Level.SEVERE, "deletePrincipal(Principal principal)", e);
				throw new AuthenticationException(e);
			}
		}
		
		// if the database update is right, we can update the in memory reference
		Principal oldPal = (Principal)localPrincipals.remove(principal.getName());
		localPrincipalsSet.remove(oldPal);
	}

}
