/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.ext.authorization;

import java.lang.reflect.Array;
import java.net.URL;
import java.security.AccessControlContext;
import java.security.AccessControlException;
import java.security.AccessController;
import java.security.CodeSource;
import java.security.DomainCombiner;
import java.security.Permission;
import java.security.Principal;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;
import java.security.ProtectionDomain;
import java.security.cert.Certificate;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.security.auth.Subject;

import net.sf.jguard.core.principals.RolePrincipal;
import net.sf.jguard.ext.authorization.domaincombiners.RestrictDomainCombiner;
import net.sf.jguard.ext.authorization.domaincombiners.StackSubjectDomainCombiner;


/**
 * utility class for authorization work related to AccessControlContext and ProtectionDomain.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 * @author <a href="mailto:vberetti@users.sourceforge.net">Vincent Beretti</a>
 * @see java.security.AccessControlContext
 * @see java.security.ProtectionDomain
 */
public class AuthorizationUtils {
	private static final Logger logger = Logger.getLogger(AuthorizationUtils.class.getName());

	/**
	 * @param subj
	 * @param p
	 * @return <code>true</code> if subject implies the permission p, <code>false</code> otherwise
	 */
	static public boolean permitted(Subject subj, final Permission p) {

	      try {
	          Subject.doAsPrivileged(subj, new PrivilegedExceptionAction() {
	              public Object run() {
	                  AccessController.checkPermission(p);
	                  // the 'null' tells the SecurityManager to consider this resource access
	                  //in an isolated context, ignoring the permissions of code currently
	                  //on the execution stack.
	                  return null;
	              }
	              },null);
	          return true;
	      } catch (AccessControlException ace) {
	    	  if(logger.isLoggable(Level.FINEST)){
	    		  logger.log(Level.FINEST,"AccessControlException ",ace);
	    	  }
	          return false;
	      } catch (PrivilegedActionException pae) {
	    	  if(logger.isLoggable(Level.FINEST)){
	    		  logger.log(Level.FINEST,"PrivilegedActionException ",pae);
	    	  }
	          return false;
	      }
	  }



	 /**
	  * return the convenient {@link AccessControlContext} corresponding to the principal.
	  * @param principal RolePrincipal used to restrict execution code rights
	  * @return object embedding used to restrict permissions
	  */
	  public static AccessControlContext getRestrictedAccessControlContext(Principal principal){
          ProtectionDomain pd = getEmptyProtectionDomain(principal);
          DomainCombiner restrictDomainCombiner = new RestrictDomainCombiner();
          AccessControlContext acc = new AccessControlContext(new ProtectionDomain[]{pd});
          AccessControlContext acc2 = new AccessControlContext(acc,restrictDomainCombiner);
		  return acc2;
	  }

	  /**
	   * gets an <code>AccessControlContext</code> containing a single <code>ProtectionDomain</code>
	   * with an <code>null</code> <code>CodeSource</code>, an empty array of <code>Certificates</code>,
	   * the current <code>Thread</code> <code>ClassLoader</code>, and the subject principals
	   * @param subject
	   * @return the generated AccessControlContext
	   */
	  public static AccessControlContext getSubjectOnlyAccessControlContext(Subject subject){
		  ProtectionDomain pd = new ProtectionDomain(new CodeSource((URL)null,(Certificate[])null),null,Thread.currentThread().getContextClassLoader(),(Principal[])subject.getPrincipals().toArray(new Principal[subject.getPrincipals().size()]));
		  ProtectionDomain[] pds = new ProtectionDomain[1];
		  pds[0] = pd;
		  AccessControlContext acc = new AccessControlContext(pds);
		  return acc;
	  }

	  public static AccessControlContext  getStackSubjectAccessControlContext(Subject subject){
		  ProtectionDomain[] arrayPd = new ProtectionDomain[0];
		  AccessControlContext acc = new AccessControlContext(arrayPd);
		  DomainCombiner dc = new StackSubjectDomainCombiner(subject);
		  AccessControlContext acc2 = new AccessControlContext(acc,dc);
		  return acc2;
	  }



	  /**
	   * return the corresponding protectionDomain mapping permissions from principal.
	   * @param principal
	   * @return an empty ProtectionDomain with the principal
	   */
	  private static ProtectionDomain getEmptyProtectionDomain(Principal principal) {
		  URL url = null;
		  Certificate[] certs = null;
		  CodeSource cs = new CodeSource(url,certs);
		  Principal[] array = (Principal[])Array.newInstance(principal.getClass(),1);
		  Array.set(array,0,principal);
		  ProtectionDomain pd = new ProtectionDomain(cs,null,Thread.currentThread().getContextClassLoader(),array);
		  return pd;
	  }

	  /**
	   * return the corresponding protectionDomain mapping permissions from principal.
	   * @param principals
	   * @return an empty ProtectionDomain with the principals
	   */
	  private static ProtectionDomain getEmptyProtectionDomain(Principal[] principals) {
		  URL url = null;
		  Certificate[] certs = null;
		  CodeSource cs = new CodeSource(url,certs);
		  ProtectionDomain pd = new ProtectionDomain(cs,null,Thread.currentThread().getContextClassLoader(),principals);
		  return pd;
	  }

	  /**
	   * return the convenient {@link AccessControlContext} corresponding to the principal.
	   * @param principals RolePrincipal sued to restrict execution code rights
	   * @return object embedding used to restrict permissions
	   */
	  public static AccessControlContext getAccessControlContext(Collection principals){
		  Iterator itPrincipals = principals.iterator();
		  Collection protectionDomains = new ArrayList();
		  while(itPrincipals.hasNext()){
			  RolePrincipal principal = (RolePrincipal)itPrincipals.next();
			  protectionDomains.add(getEmptyProtectionDomain(principal));
		  }
		  AccessControlContext acc = new AccessControlContext((ProtectionDomain[]) protectionDomains.toArray(new ProtectionDomain[protectionDomains.size()]));
		  return acc;
	  }

}
