/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.ext.authorization.manager;

import java.security.Permission;
import java.security.PermissionCollection;
import java.security.Principal;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;

import net.sf.jguard.core.authorization.manager.PermissionProvider;
import net.sf.jguard.core.authorization.permissions.Domain;
import net.sf.jguard.core.authorization.permissions.JGPermissionCollection;
import net.sf.jguard.ext.authorization.AuthorizationException;

/**
 * retrieve user's permissions.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 * @author <a href="mailto:vinipitta@users.sourceforge.net">VinÃ­cius Pitta Lima de AraÃºjo</a>
 * @author <a href="mailto:tandilero@users.sourceforge.net">Maximiliano Batelli</a>
 */
public interface AuthorizationManager extends PermissionProvider {

    /**
     * init AuthorizationManager implementation with a parameters map
     * @param options
     */
    public void init(Map options);


    public PermissionCollection getPermissionCollection(Set principals);

    /**
     * return needed initialization parameters.
     * @return parameters list.
     */
    public  List getInitParameters();

    /**
     * refresh principals and permissions data.
     */
    public void refresh();

    /**
     * create an URLPermission giving a url and a domain
     * @param url
     * @param domainName
     * @throws AuthorizationException
     */
    public void createPermission(Permission url,String domainName) throws AuthorizationException;

    public Permission readPermission(String permissionName)throws AuthorizationException;

    public void updatePermission (String oldPermissionName, Permission url,String newDomainName) throws AuthorizationException;

    public void deletePermission (String permissionName)throws AuthorizationException;

    public JGPermissionCollection listPermissions();

    public void createDomain(String domainName) throws AuthorizationException;

    public Domain readDomain(String domainName)throws AuthorizationException;

    public void updateDomain (String newName,String oldName) throws AuthorizationException;

    public void deleteDomain (String domainName)throws AuthorizationException;

    public Set listDomains()throws AuthorizationException;

    public void createPrincipal(Principal principal)throws AuthorizationException;

    /**
     * Clone a Principal with a random name
     * @param roleName Principal name to clone
     * @return cloned Principal with a different name: roleName + Random integer betweeen 0 and 99999
     * @throws AuthorizationException
     */
    public Principal clonePrincipal(String roleName) throws AuthorizationException;
    /**
     * Clone a Principal. If Principal is instance of RolePrincipal makes a call to the clone method leting the clone task to RolePrincipal
     * @param roleName Principal name to clone
     * @param cloneName Principal cloned name
     * @return cloned Principal with the given cloneName
     * @throws AuthorizationException
     */
    public Principal clonePrincipal(String roleName, String cloneName) throws AuthorizationException;

    public Principal readPrincipal(String roleName)throws AuthorizationException;

    /**
     * update the application Principal (role).
     * @param oldPrincipalName the name the principal had
     * @param principal the new principal updated
     * @see net.sf.jguard.ext.authorization.manager.AuthorizationManager#updatePrincipal(net.sf.jguard.core.principals.RolePrincipal)
     * @throws AuthorizationException
     */
    public void updatePrincipal(String oldPrincipalName, Principal principal) throws AuthorizationException;

    public void deletePrincipal (Principal principal)throws AuthorizationException;

    public Set listPrincipals();

    public Set getDomains (Collection domainNames);

    public Set getPermissions (Collection permissionNames);

    public void addToPrincipal(String roleName, Permission perm)throws AuthorizationException;

    public void addToPrincipal(String roleName, Domain domain)throws AuthorizationException;

    /* RBAC Role General Hierarchical model specific methods */

    /**
     * This commands establishes a new immediate inheritance relationship
     * between the existing principals roleAsc and the roleDesc.
     * The command is valid if and only if the role roleAsc is not an immediate
     * ascendant of roleDesc, and descendant does
     * not properly inherit roleAsc role (in order to avoid cycle creation).
     *
     * @param roleAscName  the role that will inherite.
     * @param roleDescName the role that will be inherited.
     * @throws AuthorizationException if the inheritance already exists or create a cycle.
     */
    public void addInheritance(String roleAscName, String roleDescName) throws AuthorizationException;

    /**
     * Delete the existing inheritance beteween roleAsc and roleDesc.
     *
     * @param roleAscName
     * @param roleDescName
     * @throws AuthorizationException
     */
    public void deleteInheritance(String roleAscName, String roleDescName) throws AuthorizationException;

    /**
     * replace the inital principal with the new one.
     * @param principal RolePrincipal updated
     * @throws AuthorizationException
     * @see net.sf.jguard.ext.authorization.manager.AuthorizationManager#updatePrincipal(net.sf.jguard.core.principals.RolePrincipal)
     */
    public void updatePrincipal(Principal principal) throws AuthorizationException;


    /**
     * return an unmodifiable Domain Set.
     * @return
     */
	public Set getDomainsSet();

    /**
     * return an unmodifiable Principal Set.
     * @return
     */
	public Set getPrincipalsSet();

	/**
     * return an unmodifiable Permissions Set.
     * @return
     */
	public Set getPermissionsSet();

	 public boolean isEmpty();
}
