/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/

package net.sf.jguard.ext.database;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.ext.SecurityConstants;


/**
 * Factory for Database Connection.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 * @version $Revision: 202 $
 */
public final class ConnectionFactory  {
	/** Logger for this class */
	private static final Logger logger = Logger.getLogger(ConnectionFactory.class.getName());

    private  Class clazz = null;
    private  String url = null;
    private  String login = null;
    private  String password = null;

    /**
     * key string which contains applicationName, and, if this application is "secured" (in a jGuard notion),
     * "|" and the applicationPassword
     */
    private  String key = null;
    /**
     * jndi <strong>true</strong> if the connection is reachable with the jndi abstraction,
     *  <strong>false</strong> if we should reach connection through the DriverManager
     */
    private  boolean JNDI = false;
    //Map which contains Datasources ans connection informations
    //(when the connection is reach through the driverManager)
    private  Map connMap = new HashMap();
    private  DataSource datasource = null;

    /**
     * initialise connection settings for the specified webapp in the map (with the <strong>applicationName</strong> parameter).
     * @param opts
     */
    public ConnectionFactory(Map opts){

    	logger.finest(" ConnectionFactoryOptions="+opts);

        String applicationName= (String)opts.get(CoreConstants.APPLICATION_NAME);
        String applicationPassword = (String)opts.get(SecurityConstants.APPLICATION_PASSWORD);
        if((String)opts.get(SecurityConstants.SECURED)!=null &&((String)opts.get(SecurityConstants.SECURED)).equals("true")){
         key = new  StringBuffer(applicationName).append("|").append(applicationPassword).toString();
        }else{
            key = new  String(applicationName);
        }

        String jndiRef = (String)opts.get("JNDI");
        if (jndiRef!= null){

           try {
              Hashtable env = new Hashtable(opts);
              InitialContext initCtx = new InitialContext(env);
              Object object = (DataSource) initCtx.lookup(jndiRef);
               if (object instanceof DataSource) {
				datasource= (DataSource) object;
				}else{
					throw new IllegalArgumentException(" JNDI lookup "+jndiRef+" must return an object of type javax.sql.DataSource ");
				}
              connMap.put(key,datasource);
              JNDI= true;
           } catch (NamingException e) {
				if (logger.isLoggable(Level.FINEST)) {
					logger.finest("init() -  datasource cannot be retrieved through JNDI "
                    + e.getMessage());
				}
           }catch(Throwable t){
        	   System.out.println(t.getMessage());
           }

        }else{

            String driver = (String)opts.get(SecurityConstants.DATABASE_DRIVER);
            String url =  (String)opts.get(SecurityConstants.DATABASE_DRIVER_URL);
            String login = (String)opts.get(SecurityConstants.DATABASE_DRIVER_LOGIN);
            String password = (String)opts.get(SecurityConstants.DATABASE_DRIVER_PASSWORD);
            String value = driver+"|"+url+"|"+login+"|"+password;
            connMap.put(key,value);
        }

    }

    /**
     * Returns a database connection, encapsulating the DAO details.
     *
     * @return java.sql.Connection obtained either from a pool or explicitly
     */
    public final Connection getConnection()   {
        Connection conn = null;

        //jndi stuff
        if(JNDI){
          conn = getConnectionWithJNDI();
        //driver manager stuff
        }else{
        	conn = getConnectionWithDriver();
        }
        return conn;


    }


    /**
     * Returns a database connection.this method is used to authenticate user through
     * the Jdbc Driver.
     * @param user user database connection
     * @param password password database connection
     * @return java.sql.Connection obtained either from a pool or explicitly
     */
    public final Connection getConnection(String user,String password)   {

        	Connection conn = null;
			try {
				if(url == null){
					logger.severe(" jdbc driver url is not defined ");
				}
				if(this.clazz == null){
					logger.severe(" jdbc driver class is not defined ");
				}
				conn = DriverManager.getConnection(this.url,user,password);
			} catch (SQLException e) {
                logger.severe(" connection canot be established url="+url+"  user="+user+" password="+password);
			}
        	return conn;


    }

    /**
     * get a connection grabbed through a Datasource reached by JNDI.
     * @see javax.sql.DataSource
     * @return Connection
     */
	private  Connection getConnectionWithJNDI() {
		Connection conn = null;
          try {
              conn = datasource.getConnection();
          } catch (SQLException e) {
				if (logger.isLoggable(Level.SEVERE)) {
					logger.severe("getConnection() - connection through JNDI cannot be established "
					+ e.getMessage());
				}
          }
		return conn;
	}

	/**
	 * get a connection grabbed through a Driver interface.
	 * @see java.sql.Driver
	 * @return Connection
	 */
	private Connection getConnectionWithDriver() {
		Connection conn = null;
		try {
			if(clazz==null && url==null && login==null && password==null){
				String value = (String)connMap.get(key);
				List strings = Arrays.asList(value.split("\\|"));
				conn = getConnection(strings);
			}else{
				conn = DriverManager.getConnection(url,login,password);
			}
		}catch (ClassNotFoundException e) {
				logger.severe("getConnection() - ClassNotFoundException"	+ e.getMessage());
			    logger.severe("getConnection(String, boolean)" + e);
		}catch (SQLException e) {
					logger.severe("getConnection() - SQLException "+ e.getMessage());
					logger.severe("getConnection() - SQLException state="+ e.getSQLState());
					logger.severe("getConnection() - SQLException error code="+ e.getErrorCode());
					logger.severe("getConnection() - SQLException error next exception="+ e.getNextException());
					logger.log(Level.SEVERE, "getConnection(String, boolean)", e);
		}
		return conn;
	}

	private Connection getConnection(List strings) throws ClassNotFoundException, SQLException {
		Connection conn;
		String driver = (String)strings.get(0);
		url = (String)strings.get(1);
		login = (String)strings.get(2);

		switch(strings.size()){
		   case 4:
			   //'regular' use case
			   password = (String)strings.get(3);
			   break;
		   case 3:
			   //'goofy' use case  ;-)
			   //if the list lacks one element, we implied the
			   //password is empty
			   password ="";
			   break;
		   default:
			   throw new IllegalArgumentException(" wrong number of parameters to establish JDBC connection "+strings);
		}



		if (logger.isLoggable(Level.FINEST)) {
			logger.finest("getConnection() - driver=" + driver);
			logger.finest("getConnection() - url=" + url);
			logger.finest("getConnection() - login=" + login);
			logger.finest("getConnection() - password=" + password);
		}


		Class.forName(driver);
		conn = DriverManager.getConnection(url,login,password);
		return conn;
	}




}
