/*
 jGuard is a security framework based on top of jaas (java authentication and authorization security).
 it is written for web applications, to resolve simply, access control problems.
 version $Name$
 http://sourceforge.net/projects/jguard/

 Copyright (C) 2004  Charles GAY

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 jGuard project home page:
 http://sourceforge.net/projects/jguard/

 */
package net.sf.jguard.ext.util;

import java.util.HashSet;
import java.util.Set;
import java.util.logging.Logger;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.SearchResult;

import net.sf.jguard.core.authentication.credentials.JGuardCredential;
import net.sf.jguard.core.principals.RolePrincipal;


public class JNDIUtils {
	private static final Logger logger = Logger.getLogger(JNDIUtils.class.getName());

	/**
	 * grab the name in the namespace, and return the nth value in the naming hierarchy.
	 *
	 * @param result
	 *            result found
	 * @param level
	 *            level to be retrun from the result, to the root
	 * @return value
	 */
	public static String getValueInNameSpace(SearchResult result, int level) {
		String value = null;
		String nameSpace = result.getNameInNamespace();
		String[] tokens = nameSpace.split(",");
		String[] tok = tokens[level].split("=");
		value = tok[1];
		return value;

	}

	/**
	 * prevent LDAP injection. method extracted from a CORSAIRE white paper.
	 *
	 * @param name
	 * @return safe login
	 */
	public static String escapeDn(String name) {
		// from RFC 2253 and the / character for JNDI
		final char[] META_CHARS = { '+', '"', '<', '>', ';', '/' };
		String escapedString = new String(name);
		// BackSlash is both a Java and an LDAP escape character, so escape it first
		escapedString = escapedString.replaceAll("\\\\", "\\\\");

		// positional characters - see RFC 2253
		escapedString = escapedString.replaceAll("^#", "\\\\#");
		escapedString = escapedString.replaceAll("^ | $", "\\\\ ");

		for (int i = 0; i < META_CHARS.length; i++) {
			escapedString = escapedString.replaceAll("\\" + META_CHARS[i], "\\\\" + META_CHARS[i]);
		}
		return escapedString;
	}

	/**
	 * prevent LDAP injection. method extracted from a CORSAIRE white paper.
	 *
	 * @param filterExpression
	 * @return safe filterExpression
	 */
	public static String escapeSearchFilter(String filterExpression) {
		// from RFC 2254
		String escapedString = new String(filterExpression);
		escapedString = escapedString.replaceAll("\\\\", "\\\\5c");
		escapedString = escapedString.replaceAll("\\*", "\\\\2a");
		escapedString = escapedString.replaceAll("\\(", "\\\\28");
		escapedString = escapedString.replaceAll("\\)", "\\\\29");
		return escapedString;
	}

	/**
	 * build a {@link net.sf.jguard.core.authentication.credentials.JGuardCredential} set from JNDI
	 * attributes of the user found.
	 * @param results
	 *            JNDI attributes
	 * @return set built
	 * @throws NamingException
	 */
	public static Set grabCredentialsFromJNDIAttributes(NamingEnumeration results) throws NamingException {
		Set attributes = new HashSet();
		try {
			while (results.hasMore()) {

				Attribute att = (Attribute) results.next();

				String attributeId = att.getID();
				String attributeValue = getAttributeValue(att);
				JGuardCredential credential = new JGuardCredential();
				credential.setId(attributeId);
				credential.setValue(attributeValue.toString());
				// credentials from JNDI are private
				attributes.add(credential);
			}
		} finally {
			results.close();
		}
		return attributes;
	}

	/**
	 * return as a String the attribute Value, and convert a byte[] into a new String.
	 *
	 * @param att attribute
	 * @return attribute value
	 * @throws NamingException
	 */
	private static String getAttributeValue(Attribute att) throws NamingException {
		NamingEnumeration nameEnum = null;
		StringBuffer attributeValue = new StringBuffer();
		nameEnum = att.getAll();

		int i = 0;

		while (nameEnum.hasMore()) {
			if (i != 0) {
				attributeValue.append(",");
			}
			Object obj = (Object) nameEnum.next();
			if (obj instanceof byte[]) {
				byte[] bytes = (byte[]) obj;
				obj = new String(bytes);
			}

			attributeValue.append(obj.toString());
			i++;
		}

		nameEnum.close();

		return attributeValue.toString();
	}

	/**
	 * result of an LDAP search is converted into RolePrincipal set.
	 *
	 * @param results
	 * @param ppalNameAttribute
	 * @param ppalAppNameLevel
	 * @return Set of principals
	 * @throws NamingException
	 */
	public static Set grabPrincipalsFromUser(NamingEnumeration results, String ppalNameAttribute, int ppalAppNameLevel)
			throws NamingException {
		Set principals = new HashSet();
		NamingEnumeration results2 = null;
		try {

			while (results.hasMore()) {
				SearchResult result = (SearchResult) results.next();
				logger.finest("name in namespace" + result.getNameInNamespace());
				String applicationName = JNDIUtils.getValueInNameSpace(result, ppalAppNameLevel);
				logger.finest("applicationName=" + applicationName);
				Attributes attributes = result.getAttributes();
				results2 = attributes.getAll();
				RolePrincipal principal = new RolePrincipal();
				principal.setApplicationName(applicationName);
				while (results2.hasMoreElements()) {
					Attribute att = (Attribute) results2.next();
					String attributeID = att.getID();
					String attributeValue = getAttributeValue(att);
					logger.finest("principal attributeID=" + attributeID);
					logger.finest("principal attributeValue=" + attributeValue);

					if (attributeID.equals(ppalNameAttribute)) {
						principal.setName(attributeValue);
						logger.finest("principal name=" + attributeValue);
					}
				}

				principals.add(principal);
			}
		} finally {
			results.close();
			if (results2 != null) {
				results2.close();
			}
		}
		return principals;
	}
}
