/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name:  $
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.ext.util;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.sf.jguard.core.principals.RolePrincipal;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;

/**
 * Utility class to handle XML documents with DOM4J.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 *
 */
public class XMLUtils {

	private static final String UTF_8 = "UTF-8";


	private static Logger logger = Logger.getLogger(XMLUtils.class.getName());


	private static final String DOUBLE_ANTI_SLASH = "\\\\";
	private static final String SLASH = "/";
	private static Pattern doubleAntiSlashPattern = Pattern.compile(XMLUtils.DOUBLE_ANTI_SLASH);

	private static final String ESCAPED_BLANK = "%20";
	private static final String BLANK = " ";
	private static Pattern blankPattern = Pattern.compile(XMLUtils.BLANK);

	private static final String FILE_SLASH_AND_PLUS = "file:/+";
	private static Pattern slashAndPlusPattern = Pattern.compile(XMLUtils.FILE_SLASH_AND_PLUS);
	private static final String FILE_AND_SIX_SLASH = "file://////";

	/**
	* read the xml data storage file for users and associated principals.
	* @param location
	* @return xml content
	*/
	public static Document read(String location) {

		location = resolveLocation(location);

		Document document = null;
		URL url = null;
        try {
            url = new URL(location);
        } catch (MalformedURLException e) {
			logger.severe("read(String) : " + e);
			if (logger.isLoggable(Level.FINEST)) {
				logger.finest("read() -  the 'fileLocation' value ( " + location
						+ " ) present in the web.xml file is a malformed URL ");
			}
        }
        SAXReader reader = new SAXReader(true);
        try {
             document = reader.read(url);
        } catch (DocumentException e1) {
			logger.severe("read(String) : " + e1);
        }
        if(document==null){
        	throw new IllegalArgumentException(location+" does not point to an XML document ");
        }
        return document;
	}

	/**
	 * resolve cleanly location with the 'file' protocol.
	 * @param location
	 * @return location clean
	 */
	public static String resolveLocation(String location) {

		//remove trailing space character
		location = location.trim();

		//replace remaining blank character by '%20'
        Matcher blankMatcher = blankPattern.matcher(location);
        location = blankMatcher.replaceAll(XMLUtils.ESCAPED_BLANK);

        //replace '\\' by '/' (avoid windows stuff)
		Matcher antiSlashMatcher = doubleAntiSlashPattern.matcher(location);
		location = antiSlashMatcher.replaceAll(XMLUtils.SLASH);

		 //it should be a Windows Operating system
		 //if it does not start with the pattern 'file:'
		if(!location.startsWith("file:")){
        	location = new StringBuffer("file://////").append(location).toString();
        }

		//replace any different pattern not like 'file://////' by this pattern
        if(!location.matches(XMLUtils.FILE_AND_SIX_SLASH)){
        	Matcher slashAndPlus = slashAndPlusPattern.matcher(location);
        	location = slashAndPlus.replaceFirst(XMLUtils.FILE_AND_SIX_SLASH);
        }

        if (logger.isLoggable(Level.FINEST)) {
			logger.finest("location="+location);
    	}
		return location;
	}

	public static void write(String fileLocation,Document document) throws IOException{
		XMLUtils.write(new URL(fileLocation),document);
	}

	/**
	 * write the updated configuration to the XML file in the UTF-8 format.
	 * @param url - URL of the file to write
	 * @param document dom4j Document
	 * @throws IOException
	 */
	public static void write(URL url,Document document) throws IOException{
	    OutputFormat outFormat = OutputFormat.createPrettyPrint();
	    outFormat.setEncoding(UTF_8);
	    XMLWriter out = new XMLWriter(new FileWriter(new File(url.getPath())),outFormat);
	    out.write(document);
	    out.flush();
	    out.close();
	}

	 /**
     * remove in the ascendants elements all the <i>principalRef</i> references with the name of the
     * principal.
     * @param root DOM4J Element of the document
     * @param principal
     */
    public static  void deletePrincipalRefs(Element root, RolePrincipal principal){
    	Element principalsElement = root.element("principals");
        List principalsRefList = principalsElement.selectNodes("//principalRef[@name='"+principal.getName()+"']");
        Iterator descendantsRefIt = principalsRefList.iterator();
        while(descendantsRefIt.hasNext()){
        	Element principalRef = (Element)descendantsRefIt.next();
        	Element principalElement = principalRef.getParent();
        	principalElement.remove(principalRef);
        	if(principalElement.elements("principalRef").size()==0){
        		principalElement.getParent().remove(principalElement);
        	}
        }
    }

}
